#!/usr/bin/env nomsu -V7.0.0-- File lib/core/metaprogramming.nom chunk #1
-- 
-- This File contains actions for making actions and compile-time actions and some helper
-- functions to make that easier.
-- ;
do
    local mangle_index = 0
    function mangler()
        local my_mangle_index = mangle_index
        mangle_index = mangle_index + 1
        return function(varname)
            return (varname..(("\3%X"):format(my_mangle_index))):as_lua_id()
        end
    end
end
COMPILE_RULES["define mangler"] = function(_ENV, _tree)
    return LuaCode("local mangle = mangler()")
end
COMPILE_RULES["this tree"] = function(_ENV, _tree)
    return LuaCode("_tree")
end
COMPILE_RULES["1 ->"] = function(_ENV, _tree, args, body)
    if not args and not body then args, body = {}, SyntaxTree{type='Action', "do", "nothing"}
    elseif args and not body then args, body = {}, args end
    local body_lua = SyntaxTree:is_instance(body) and _ENV:compile(body) or body
    if SyntaxTree:is_instance(body) and body.type ~= "Block" then body_lua:prepend("return ") end
    local lua = LuaCode("(function(")
    if SyntaxTree:is_instance(args) and (args.type == "Action" or args.type == "MethodCall") then
        args = args:get_args()
    elseif SyntaxTree:is_instance(args) and args.type == "Var" then args = {args} end
    for i, arg in ipairs(args) do
        local arg_lua = SyntaxTree:is_instance(arg) and _ENV:compile(arg):text() or arg
        if arg_lua == "..." then
            if i < #args then
                at_1_fail(SyntaxTree:is_instance(arg) and arg or nil,
                    "Compile error: Extra arguments must come last. "..
                    "Hint: Try removing any arguments after (*extra arguments*)")
            end
        elseif not arg_lua:is_lua_id() then
            at_1_fail(SyntaxTree:is_instance(arg) and arg or nil,
                "Compile error: This does not compile to a Lua identifier ("..arg_lua.."),"..
                "so it can't be used as a function argument. "..
                "Hint: This should probably be a Nomsu variable instead (like $x).")
        end
        lua:add(i > 1 and ", " or "", arg_lua)
        body_lua:remove_free_vars({arg_lua})
    end
    body_lua:declare_locals()
    lua:add(")\n    ", body_lua, "\nend)")
    return lua
end
COMPILE_RULES["->"] = COMPILE_RULES["1 ->"]
COMPILE_RULES["for"] = COMPILE_RULES["1 ->"]

COMPILE_RULES["\\"] = function(_ENV, _tree, escaped)
    local function escape(t)
        if t.type == "Action" and t:get_stub() == "\\" and #t == 2 then
            return _ENV:compile(t[2])
        else
            local bits = {}
            table.insert(bits, "type="..t.type:as_lua())
            if t.source then
                table.insert(bits, "source="..t.source:as_lua())
            end
            for i,b in ipairs(t) do
                table.insert(bits, lua_type_of(b) == 'string' and b:as_lua() or escape(b))
            end
            local lua = LuaCode:from(t.source, "SyntaxTree{")
            lua:concat_add(bits, ", ")
            lua:add("}")
            return lua
        end
    end
    return escape(escaped)
end

-- File lib/core/metaprogramming.nom chunk #2
TESTS["@lib/core/metaprogramming.nom[3650:3678]"] = "\n(five) compiles to \"5\"";
TESTS["@lib/core/metaprogramming.nom[3684:3796]"] = "\nunless ((five) == 5):\n    fail \"Compile to expression failed.\"\n(loc x) compiles to \"local x = 99;\"";
TESTS["@lib/core/metaprogramming.nom[3802:4076]"] = "\nlua> \"do\"\nloc x\nunless ($x == 99):\n    fail \"Compile to statements with locals failed.\"\nlua> \"end\"\nunless ($x == (nil)):\n    fail \"Failed to properly localize a variable.\"\n\n(asdf) compiles to:\n    $tmp = \"\"\n    return (Lua $tmp)";
TESTS["@lib/core/metaprogramming.nom[4082:4167]"] = "\nasdf\nunless ($tmp == (nil)):\n    fail \"compile to is leaking variables\"";
COMPILE_RULES["1 compiles to"] = function(_ENV, _tree, action, body)
    local args = a_List{"_ENV", "_tree"}
    if body.type == "Text" then
        body = SyntaxTree{source=body.source, type="Action", "Lua", body}
    end
    if not (action.type == "Action" or
            (action.type == "EscapedNomsu" and action[1].type == "Action") or
            action.type == "MethodCall") then
        at_1_fail(action.source, "Compile error: "..
            "This first argument to (* compiles to *) is neither an action nor an escaped action (it's a "..action.type.."). "..
            "Hint: This should probably be an action like:\n"
            .."(foo $x) compiles to \"(\\($x as lua) + 1)\"")
    end
    if action.type == "EscapedNomsu" then
        for _,a in ipairs(action[1]) do
            if a.type == "EscapedNomsu" then args:add(a[1]) end
        end
        return LuaCode("COMPILE_RULES[", _ENV:compile(action), ":get_stub()] = ",
            _ENV:compile(SyntaxTree{type="Action", source=Source("lib/core/metaprogramming.nom", 5298, 5314), args, "->", body}))
    else
        for _,a in ipairs(action:get_args()) do args:add(a) end
        return LuaCode("COMPILE_RULES[", action:get_stub():as_lua(),
            "] = ", _ENV:compile(SyntaxTree{type="Action", source=Source("lib/core/metaprogramming.nom", 5513, 5529), args, "->", body}))
    end
end

-- File lib/core/metaprogramming.nom chunk #3
COMPILE_RULES["1 all compile to"] = (function(_ENV, _tree, actions, body)
    if actions.type ~= "List" then
        at_1_fail(actions, "Compile error: This should be a list of actions.")
    end
    local lua = _ENV:compile(SyntaxTree{type="Action", source=Source("lib/core/metaprogramming.nom", 5865, 5895), actions[1], "compiles", "to", body})
    local args = a_List{"_ENV", "_tree", unpack(actions[1]:get_args())}
    local compiledx5Fargs = a_List{"_ENV", "_tree"};
    for i=3,#args do compiledx5Fargs[i] = _ENV:compile(args[i]) end
    for i=2,#actions do
        local alias = actions[i]
        local aliasx5Fargs = a_List{"_ENV", "_tree", unpack(alias:get_args())}
        lua:add("\nCOMPILE_RULES[", alias:get_stub():as_lua(), "] = ")
        if aliasx5Fargs == args then
            lua:add("COMPILE_RULES[", actions[1]:get_stub():as_lua(), "]")
        else
            lua:add("function(")
            local compiledx5Faliasx5Fargs = a_List{"_ENV", "_tree"};
            for i=3,#aliasx5Fargs do compiledx5Faliasx5Fargs[i] = _ENV:compile(aliasx5Fargs[i]) end
            lua:concat_add(compiledx5Faliasx5Fargs, ", ")
            lua:add(") return COMPILE_RULES[", actions[1]:get_stub():as_lua(), "](")
            lua:concat_add(compiledx5Fargs, ", ")
            lua:add(") end")
        end
    end
    return lua;
end);

-- File lib/core/metaprogramming.nom chunk #4
TESTS["@lib/core/metaprogramming.nom[7200:7571]"] = "\n(foo $x) means \"outer\"\nwith [$(foo $)]:\n    (foo $x) means:\n        $y = ($x + 1)\n        return $y\n    \n    unless ((foo 10) == 11):\n        fail \"Action didn't work.\"\n    \n    unless ($y == (nil)):\n        fail \"Action leaked a local into globals.\"\n    \n    (baz $) parses as (foo $)\nassume ((foo 1) == \"outer\")";
COMPILE_RULES["1 means"] = (function(_ENV, _tree, action, body)
    local lua = LuaCode()
    if action.type == "MethodCall" then
        lua:add(_ENV:compile(action[1]), ".", action[2]:get_stub():as_lua_id())
    elseif action.type == "Action" then
        lua:add(action:get_stub():as_lua_id())
        lua:add_free_vars({action:get_stub():as_lua_id()})
    else
        at_1_fail(action, "Compile error: This is not an action or method call.")
    end
    lua:add(" = ", _ENV:compile(SyntaxTree{type="Action", source=Source("lib/core/metaprogramming.nom", 8104, 8122), action, "->", body}), ";")
    return lua;
end);
COMPILE_RULES["1 all mean"] = (function(_ENV, _tree, actions, body)
    local lua = _ENV:compile(SyntaxTree{type="Action", source=Source("lib/core/metaprogramming.nom", 8240, 8264), actions[1], "means", body})
    local first_def = (actions[1].type == "MethodCall"
        and LuaCode(_ENV:compile(actions[1][1]), ".", actions[1][2]:get_stub():as_lua_id())
        or LuaCode(actions[1]:get_stub():as_lua_id()))
    local args = a_List(actions[1]:get_args())
    for i=2,#actions do
        local alias = actions[i]
        local aliasx5Fargs = a_List(alias:get_args())
        lua:add("\n")
        if alias.type == "MethodCall" then
            lua:add(_ENV:compile(alias[1]), ".", alias[2]:get_stub():as_lua_id())
        else
            lua:add(alias:get_stub():as_lua_id())
            lua:add_free_vars({alias_name})
        end
        if args == aliasx5Fargs then
            lua:add(" = ", first_def, ";")
        else
            lua:add(" = ", _ENV:compile(SyntaxTree{type="Action", source=Source("lib/core/metaprogramming.nom", 9159, 9186), aliasx5Fargs, "->", actions[1]}), ";")
        end
    end
    return lua;
end);
TESTS["@lib/core/metaprogramming.nom[9261:9302]"] = "\n$loc = 99\nexternal ($glob = 99)";
TESTS["@lib/core/metaprogramming.nom[9308:9357]"] = "\nassume $loc == (nil)\nassume $glob == 99";
COMPILE_RULES["external"] = (function(_ENV, _tree, body)
    local lua = _ENV:compile(body)
    lua:remove_free_vars()
    return lua;
end);
TESTS["@lib/core/metaprogramming.nom[9499:9799]"] = "\n[$x, $y] = [\"outer\", \"outer\"]\nexternal:\n    (set external x local y) means:\n        with external [$x]:\n            $x = \"inner\"\n            $y = \"inner\"\n\nset external x local y\nunless (($x == \"inner\") and ($y == \"outer\")):\n    fail \"'with external' failed.\"";
COMPILE_RULES["with external"] = (function(_ENV, _tree, externals, body)
    local body_lua = _ENV:compile(body)
    local varnames = {}
    for i,v in ipairs(externals) do
        varnames[i] = _ENV:compile(v):text()
    end
    body_lua:remove_free_vars(varnames)
    return body_lua;
end);
TESTS["@lib/core/metaprogramming.nom[10110:10218]"] = "\n(swap $x and $y) parses as\n    do:\n        $tmp = $x\n        $x = $y\n        $y = $tmp";
TESTS["@lib/core/metaprogramming.nom[10224:10509]"] = "\n[$1, $2] = [1, 2]\nswap $1 and $2\nunless (($1 == 2) and ($2 == 1)):\n    fail \"'parse $ as $' failed on 'swap $ and $'\"\n[$tmp, $tmp2] = [1, 2]\nswap $tmp and $tmp2\nunless (($tmp == 2) and ($tmp2 == 1)):\n    fail \"'parse $ as $' variable mangling failed.\"";
COMPILE_RULES["1 all parse as"] = (function(_ENV, _tree, actions, body)
    local replacements = {}
    if actions.type ~= "List" then
        at_1_fail(actions, "Compile error: This should be a list.")
    end
    for i,arg in ipairs(actions[1]:get_args()) do
        replacements[arg[1]] = _ENV:compile(arg):text()
    end
    local function make_tree(t)
        if SyntaxTree:is_instance(t) and t.type == "Var" then
            if replacements[t:as_var()] then
                return replacements[t:as_var()]
            else
                return "SyntaxTree{mangle("..t:as_var():as_lua().."), type="..t.type:as_lua()..", source=".._1_as_text(t.source):as_lua().."}"
            end
        elseif SyntaxTree:is_instance(t) then
            local ret = {}
            local i = 1
            for k, v in pairs(t) do
                if k == i then
                    ret[#ret+1] = make_tree(t[i])
                    i = i + 1
                elseif k == "source" then
                    ret[#ret+1] = k.."= ".._1_as_text(v):as_lua()
                elseif lua_type_of(k) == 'string' and k:is_a_lua_id() then
                    ret[#ret+1] = k.."= "..make_tree(v)
                else
                    ret[#ret+1] = "["..make_tree(k).."]= "..make_tree(v)
                end
            end
            return "SyntaxTree{"..table.concat(ret, ", ").."}"
        elseif lua_type_of(t) == 'number' then
            return _1_as_text(t)
        else
            return t:as_lua()
        end
    end
    local newx5Fbody = LuaCode:from(body.source,
        "local mangle = mangler()",
        "\nreturn ", make_tree(body))
    return _ENV:compile(SyntaxTree{type="Action", source=Source("lib/core/metaprogramming.nom", 12338, 12373), actions, "all", "compile", "to", newx5Fbody});
end);

-- File lib/core/metaprogramming.nom chunk #5
COMPILE_RULES["1 parses as"] = (function(_ENV, _tree, action, body)
    local mangle = mangler()
    return SyntaxTree{SyntaxTree{action, type= "List", source= "@lib/core/metaprogramming.nom[12513:12522]"}, "all", "parse", "as", body, type= "Action", source= "@lib/core/metaprogramming.nom[12513:12541]", stub= "1 all parse as"}
end);
_ENV._1_as_lua_expr = (function(_ENV, tree)
    local tree_lua = _ENV:compile(tree)
    if tree.type == 'Block' and #tree > 1 then
        tree_lua = LuaCode:from(tree.source, '(function()\n    ', tree_lua, '\nend)()')
    end
    return tree_lua;
end);
--  Need to make sure the proper environment is used for compilation (i.e. the caller's environment);
COMPILE_RULES["1 as lua expr"] = (function(_ENV, _tree, tree)
    return SyntaxTree{type="MethodCall", source=Source("lib/core/metaprogramming.nom", 12993, 13030), SyntaxTree{type="Action", source=Source("lib/core/metaprogramming.nom", 12993, 13010), "nomsu", "environment"}, SyntaxTree{type="Action", source=Source("lib/core/metaprogramming.nom", 13012, 13030), tree, "as", "lua", "expr"}}
end);

-- File lib/core/metaprogramming.nom chunk #6
_1_as_lua_identifier = (function(var)
    local lua = _ENV:compile(var)
    if not lua:text():is_a_lua_id() then
        at_1_fail(var, "Compile error: "..
            "This is supposed to be something that compiles to a valid Lua identifier. "..
            "Hint: This should probably be a variable.")
    end
    return lua;
end);
_1_as_lua_id = _1_as_lua_identifier;
TESTS["@lib/core/metaprogramming.nom[13556:13858]"] = "\n(num args (*extra arguments*)) means #(*extra arguments*)\nassume (num args 1 2 3) == 3\n(extra args (*extra arguments*)) means [*extra arguments*]\nassume (extra args 1 2 3) == [1, 2, 3]\n(third arg (*extra arguments*)) means (*extra arguments*).3\nassume (third arg 5 6 7 8) == 7";
COMPILE_RULES["* extra arguments *"] = (function(_ENV, _tree)
    return LuaCode:from("@lib/core/metaprogramming.nom[13892:13897]", "...")
end);
COMPILE_RULES["1 is syntax tree"] = (function(_ENV, _tree, _)
    return LuaCode:from("@lib/core/metaprogramming.nom[13929:13971]",
        "SyntaxTree:is_instance(", _ENV:_1_as_lua_expr(_), ")")
end);
_1_is_2_syntax_tree = (function(_, kind)
    return SyntaxTree:is_instance(_) and _.type == kind
end);
match_1_with = (function(tree, patt)
    if patt.type == "Var" then return a_Dict{[patt:as_var()]=tree} end
    if patt.type == "Action" and patt:get_stub() ~= tree:get_stub() then return nil end
    if #patt ~= #tree then return nil end
    local matches = a_Dict{}
    for i=1,#patt do
        if SyntaxTree:is_instance(tree[i]) then
            local submatch = match_1_with(tree[i], patt[i])
            if not submatch then return nil end
            for k,v in pairs(submatch) do
                if matches[k] and matches[k] ~= v then return nil end
                matches[k] = v
            end
        end
    end
    return matches;
end);
TESTS["@lib/core/metaprogramming.nom[14912:15049]"] = "\nassume\n    (\n        quote (\"\n            one\n            \"two\"\n        \")\n    ) == \"\\\"one\\\\n\\\\\\\"two\\\\\\\"\\\"\"";
quote = (function(_)
    return (_1_as_text(_)):as_lua()
end);
TESTS["@lib/core/metaprogramming.nom[15106:15426]"] = "\nassume (lua type of {}) == \"table\"\nassume (type of {}) == \"a Dict\"\nassume ((type of 5) == \"a Number\")\nassume ({} is \"a Dict\")\nassume (\"\" is text)\nassume (\"\" is \"Text\")\nassume (5 is \"a Number\")\nassume ((->) is \"an Action\")\nassume ((yes) is \"a Boolean\")\nassume (\"\" isn't \"a Dict\")";
_1_is_text = (function(_)
    return lua_type_of(_) == 'string'
end);
_1_is_not_text = (function(_)
    return lua_type_of(_) ~= 'string'
end);
_1_isn_x27_t_text = _1_is_not_text;
type_of = (function(_)
    local mt = getmetatable(_)
    if mt and mt.__type then return mt.__type end
    if _ == nil then return 'nil' end
    local lua_type = lua_type_of(_)
    return 'a '..lua_type:capitalized();
end);
_1_is = (function(_, type)
    local class = getmetatable(_)
    ::check_parent::
    if not class or not class.__type then return 'a '..lua_type_of(_):capitalized() == type end
    if class.__type == type then return true end
    local class_mt = getmetatable(class)
    if class_mt.__index and class_mt.__index ~= class then
        class = class_mt.__index
        goto check_parent
    end
    return false;
end);
COMPILE_RULES["1 isn ' t"] = (function(_ENV, _tree, _, type)
    local mangle = mangler()
    return SyntaxTree{"not", SyntaxTree{_, "is", type, type= "Action", source= "@lib/core/metaprogramming.nom[16447:16457]", stub= "1 is"}, type= "Action", source= "@lib/core/metaprogramming.nom[16442:16458]", stub= "not"}
end)
COMPILE_RULES["1 is not"] = COMPILE_RULES["1 isn ' t"];

-- File lib/core/metaprogramming.nom chunk #7
TESTS["@lib/core/metaprogramming.nom[16547:16621]"] = "\nassume (\"Action\" tree with \"foo\" (\"Var\" tree with \"x\")) == \\(foo $x)";
_1_tree_with = (function(type, ...)
    return SyntaxTree({type=type, ...})
end);
_1_tree_from = (function(type, source)
    return SyntaxTree({type=type, source=source})
end);
_1_tree_from_2_with = (function(type, source, ...)
    return SyntaxTree({type=type, source=source, ...})
end);
TESTS["@lib/core/metaprogramming.nom[16963:17043]"] = "\n(foo) means:\n    return 100 200 300\nassume (select 2 (foo)) == 200";
--  Return statement is wrapped in a do..end block because Lua is unhappy if you
-- put code after a return statement, unless you wrap it in a block.;
COMPILE_RULES["return"] = (function(_ENV, _tree, ...)
    local lua = LuaCode:from("@lib/core/metaprogramming.nom[17276:17288]", "do return ")
    for i=1,select('#',...) do
        if i > 1 then lua:add(", ") end
        lua:add(_ENV:compile((select(i, ...))))
    end
    lua:add(" end")
    return lua;
end);
--  Convenience helper:;
COMPILE_RULES["return Lua"] = (function(_ENV, _tree, ...)
    return SyntaxTree{type="Action", source=Source("lib/core/metaprogramming.nom", 17571, 17604), "return", SyntaxTree{type="Action", source=Source("lib/core/metaprogramming.nom", 17579, 17603), "Lua", ...}}
end);
--  Literals;
COMPILE_RULES["yes"] = (function(_ENV, _tree)
    return LuaCode:from("@lib/core/metaprogramming.nom[17638:17646]", "(true)")
end);
COMPILE_RULES["no"] = (function(_ENV, _tree)
    return LuaCode:from("@lib/core/metaprogramming.nom[17664:17673]", "(false)")
end);
COMPILE_RULES["nothing"] = (function(_ENV, _tree)
    return LuaCode:from("@lib/core/metaprogramming.nom[17710:17717]", "(nil)")
end)
COMPILE_RULES["nil"] = COMPILE_RULES["nothing"]
COMPILE_RULES["null"] = COMPILE_RULES["nothing"];
COMPILE_RULES["command line args"] = (function(_ENV, _tree)
    return LuaCode:from("@lib/core/metaprogramming.nom[17750:17769]", "COMMAND_LINE_ARGS")
end);

-- File lib/core/metaprogramming.nom chunk #8
COMPILE_RULES["at compilation"] = (function(_ENV, _tree, expr)
    local value = _ENV:run(SyntaxTree{type="Action", source=Source("lib/core/metaprogramming.nom", 17952, 17965), "return", expr})
    if lua_type_of(value) == 'table' or lua_type_of(value) == 'string' and value.as_lua then
        return LuaCode(value:as_lua())
    else
        return LuaCode(tostring(value))
    end
end);
TESTS["@lib/core/metaprogramming.nom[18190:18352]"] = "\nusing compile rules:\n    (yes) compiles to \"3\"\n    assume $(COMPILE RULES).yes\n..do:\n    assume (yes) == 3\nassume (yes) == (=lua \"true\")";
COMPILE_RULES["using compile rules 1 do"] = (function(_ENV, _tree, rules, body)
    local env = new_environment()
    env:run(rules)
    local lua = env:compile(body)
    return lua;
end);

-- File lib/core/metaprogramming.nom chunk #9
--  TODO: Remove shim;
COMPILE_RULES["1 with 2 ->"] = (function(_ENV, _tree, tree, t, replacement)
    local mangle = mangler()
    return SyntaxTree{tree, SyntaxTree{"with", SyntaxTree{t, "->", replacement, type= "Action", source= "@lib/core/metaprogramming.nom[18708:18726]", stub= "1 ->"}, type= "Action", source= "@lib/core/metaprogramming.nom[18702:18727]", stub= "with"}, type= "MethodCall", source= "@lib/core/metaprogramming.nom[18695:18727]"}
end);
COMPILE_RULES["tree 1 with vars"] = (function(_ENV, _tree, tree, replacements)
    local mangle = mangler()
    return SyntaxTree{tree, SyntaxTree{"with", replacements, type= "Action", source= "@lib/core/metaprogramming.nom[18821:18839]", stub= "with"}, type= "MethodCall", source= "@lib/core/metaprogramming.nom[18814:18839]"}
end)
COMPILE_RULES["1 with vars"] = COMPILE_RULES["tree 1 with vars"];
COMPILE_RULES["1 has subtree"] = (function(_ENV, _tree, tree, matchx5Ftree)
    local mangle = mangler()
    return SyntaxTree{tree, SyntaxTree{"contains", matchx5Ftree, type= "Action", source= "@lib/core/metaprogramming.nom[18890:18910]", stub= "contains"}, type= "MethodCall", source= "@lib/core/metaprogramming.nom[18883:18910]"}
end);
