#!/usr/bin/env nomsu -V7.0.0-- File lib/core/operators.nom chunk #1
-- 
-- This file contains definitions of operators like "+" and "and".
-- ;
_ENV:use("core/metaprogramming");

-- File lib/core/operators.nom chunk #2
TESTS["@lib/core/operators.nom[221:286]"] = "\nassume (all [1 < 2, 2 > 1, 1 <= 2, 2 >= 1, 1 == 1, 1 != 2])";
--  Comparison Operators;
COMPILE_RULES["1 <"] = (function(_ENV, _tree, x, y)
    return LuaCode:from("@lib/core/operators.nom[335:376]", "(", _ENV:_1_as_lua_expr(x),
        " < ", _ENV:_1_as_lua_expr(y), ")")
end);
COMPILE_RULES["1 >"] = (function(_ENV, _tree, x, y)
    return LuaCode:from("@lib/core/operators.nom[399:440]", "(", _ENV:_1_as_lua_expr(x),
        " > ", _ENV:_1_as_lua_expr(y), ")")
end);
COMPILE_RULES["1 <="] = (function(_ENV, _tree, x, y)
    return LuaCode:from("@lib/core/operators.nom[464:506]", "(", _ENV:_1_as_lua_expr(x),
        " <= ", _ENV:_1_as_lua_expr(y), ")")
end);
COMPILE_RULES["1 >="] = (function(_ENV, _tree, x, y)
    return LuaCode:from("@lib/core/operators.nom[530:572]", "(", _ENV:_1_as_lua_expr(x),
        " >= ", _ENV:_1_as_lua_expr(y), ")")
end);
COMPILE_RULES["1 =="] = (function(_ENV, _tree, a, b)
    return LuaCode:from("@lib/core/operators.nom[596:638]", "(", _ENV:_1_as_lua_expr(a),
        " == ", _ENV:_1_as_lua_expr(b), ")")
end);
COMPILE_RULES["1 not ="] = (function(_ENV, _tree, a, b)
    return LuaCode:from("@lib/core/operators.nom[677:719]", "(", _ENV:_1_as_lua_expr(a),
        " ~= ", _ENV:_1_as_lua_expr(b), ")")
end)
COMPILE_RULES["1 !="] = COMPILE_RULES["1 not ="];

-- File lib/core/operators.nom chunk #3
TESTS["@lib/core/operators.nom[807:1166]"] = "\n$x = 10\nassume ($x == 10)\n[$x, $y] = [10, 20]\nunless (($x == 10) and ($y == 20)):\n    fail \"mutli-assignment failed.\"\n[$x, $y] = [$y, $x]\nunless (($y == 10) and ($x == 20)):\n    fail \"swapping vars failed.\"\n$vals = [4, 5]\n[$x, $y] = (unpack $vals)\nunless (($x == 4) and ($y == 5)):\n    fail \"unpacking failed\"";
--  Variable assignment operator;
COMPILE_RULES["1 ="] = (function(_ENV, _tree, var, value)
    local lua = LuaCode()
    if var.type == "List" then
        for i, assignment in ipairs(var) do
            if i > 1 then lua:add(", ") end
            local assignment_lua = _ENV:_1_as_lua_expr(assignment)
            lua:add(assignment_lua)
            if assignment.type == 'Var' and assignment[1].type ~= "MethodCall" then
                lua:add_free_vars({assignment_lua:text()})
            end
        end
        lua:add(' = ')
        if value.type == "List" then
            if #value ~= #var then
                compile_error_at(value,
                    "This assignment has too "..(#value > #var and "many" or "few").." values.",
                    "Make sure it has the same number of values on the left and right hand side of the '=' operator.")
            end
            for i, val in ipairs(value) do
                if i > 1 then lua:add(", ") end
                local val_lua = _ENV:_1_as_lua_expr(val)
                lua:add(val_lua)
            end
            lua:add(";")
        else
            lua:add(_ENV:_1_as_lua_expr(value), ';')
        end
    else
        local var_lua = _ENV:_1_as_lua_expr(var)
        lua:add(var_lua)
        if var.type == 'Var' and var[1].type ~= "MethodCall" then
            lua:add_free_vars({var_lua:text()})
        end
        lua:add(' = ', _ENV:_1_as_lua_expr(value), ';')
    end
    return lua;
end);

-- File lib/core/operators.nom chunk #4
TESTS["@lib/core/operators.nom[2867:3235]"] = "\n[$x, $y] = [1, 2]\nwith [$z, $x = 999]:\n    assume $z == (nil)\n    $z = 999\n    unless ($z == 999):\n        fail \"'with' failed.\"\n    \n    unless ($x == 999):\n        fail \"'with' assignment failed.\"\n\nunless ($x == 1):\n    fail \"'with' scoping failed\"\n\nunless ($z == (nil)):\n    fail \"'with' scoping failed\"";
COMPILE_RULES["with"] = (function(_ENV, _tree, assignments, body)
    local defs = LuaCode()
    for i, item in ipairs(assignments) do
        if i > 1 then defs:add("\n") end
        local item_lua = _ENV:compile(item)
        if item.type == 'Action' and item.stub == '1 =' then
            item_lua:remove_free_vars(item_lua.free_vars)
        end
        defs:add("local ", item_lua, ";")
    end
    do return LuaCode:from("@lib/core/operators.nom[3698:3793]", "",
        LuaCode:from("@lib/core/operators.nom[3709:3720]", "do\n"),
        LuaCode:from("@lib/core/operators.nom[3720:3724]", "    "), defs, "\n",
        LuaCode:from("@lib/core/operators.nom[3739:3743]", "    "), _ENV:compile(body),
        "\n",
        LuaCode:from("@lib/core/operators.nom[3767:3786]", "end -- 'with' block")) end
end);
--  Math Operators;
TESTS["@lib/core/operators.nom[3818:3890]"] = "\nunless ((5 wrapped around 2) == 1):\n    fail \"mod not working\"";
COMPILE_RULES["1 wrapped around"] = (function(_ENV, _tree, x, y)
    return LuaCode:from("@lib/core/operators.nom[3954:3999]", "((", _ENV:_1_as_lua_expr(x),
        ") % (", _ENV:_1_as_lua_expr(y), "))")
end)
COMPILE_RULES["1 mod"] = COMPILE_RULES["1 wrapped around"]
COMPILE_RULES["1 %"] = COMPILE_RULES["1 wrapped around"];
--  3-part chained comparisons;
--  (uses a lambda to avoid re-evaluating middle value, while still being an expression);
TESTS["@lib/core/operators.nom[4125:4419]"] = "\n$calls = 0\n(one) means:\n    external:\n        $calls = ($calls + 1)\n    return 1\n\nunless (0 <= (one) <= 2):\n    fail \"Three-way chained comparison failed.\"\n\nunless ($calls == 1):\n    fail \"Three-way comparison evaluated middle value multiple times\"";
COMPILE_RULES["1 < 2 <"] = (function(_ENV, _tree, x, y, z)
    local mangle = mangler()
    return SyntaxTree{SyntaxTree{SyntaxTree{SyntaxTree{mangle("a"), type="Var", source="@lib/core/operators.nom[4449:4451]"}, SyntaxTree{mangle("b"), type="Var", source="@lib/core/operators.nom[4452:4454]"}, SyntaxTree{mangle("c"), type="Var", source="@lib/core/operators.nom[4455:4457]"}, type= "Action", source= "@lib/core/operators.nom[4449:4457]", stub= ""}, "->", SyntaxTree{SyntaxTree{SyntaxTree{mangle("a"), type="Var", source="@lib/core/operators.nom[4464:4466]"}, "<", SyntaxTree{mangle("b"), type="Var", source="@lib/core/operators.nom[4469:4471]"}, type= "Action", source= "@lib/core/operators.nom[4464:4471]", stub= "1 <"}, "and", SyntaxTree{SyntaxTree{mangle("b"), type="Var", source="@lib/core/operators.nom[4478:4480]"}, "<", SyntaxTree{mangle("c"), type="Var", source="@lib/core/operators.nom[4483:4485]"}, type= "Action", source= "@lib/core/operators.nom[4478:4485]", stub= "1 <"}, type= "Action", source= "@lib/core/operators.nom[4463:4486]", stub= "1 and"}, type= "Action", source= "@lib/core/operators.nom[4448:4487]", stub= "1 ->"}, x, y, z, type= "Action", source= "@lib/core/operators.nom[4447:4497]", stub= ""}
end);
COMPILE_RULES["1 <= 2 <"] = (function(_ENV, _tree, x, y, z)
    local mangle = mangler()
    return SyntaxTree{SyntaxTree{SyntaxTree{SyntaxTree{mangle("a"), type="Var", source="@lib/core/operators.nom[4528:4530]"}, SyntaxTree{mangle("b"), type="Var", source="@lib/core/operators.nom[4531:4533]"}, SyntaxTree{mangle("c"), type="Var", source="@lib/core/operators.nom[4534:4536]"}, type= "Action", source= "@lib/core/operators.nom[4528:4536]", stub= ""}, "->", SyntaxTree{SyntaxTree{SyntaxTree{mangle("a"), type="Var", source="@lib/core/operators.nom[4543:4545]"}, "<=", SyntaxTree{mangle("b"), type="Var", source="@lib/core/operators.nom[4549:4551]"}, type= "Action", source= "@lib/core/operators.nom[4543:4551]", stub= "1 <="}, "and", SyntaxTree{SyntaxTree{mangle("b"), type="Var", source="@lib/core/operators.nom[4558:4560]"}, "<", SyntaxTree{mangle("c"), type="Var", source="@lib/core/operators.nom[4563:4565]"}, type= "Action", source= "@lib/core/operators.nom[4558:4565]", stub= "1 <"}, type= "Action", source= "@lib/core/operators.nom[4542:4566]", stub= "1 and"}, type= "Action", source= "@lib/core/operators.nom[4527:4567]", stub= "1 ->"}, x, y, z, type= "Action", source= "@lib/core/operators.nom[4526:4577]", stub= ""}
end);
COMPILE_RULES["1 < 2 <="] = (function(_ENV, _tree, x, y, z)
    local mangle = mangler()
    return SyntaxTree{SyntaxTree{SyntaxTree{SyntaxTree{mangle("a"), type="Var", source="@lib/core/operators.nom[4608:4610]"}, SyntaxTree{mangle("b"), type="Var", source="@lib/core/operators.nom[4611:4613]"}, SyntaxTree{mangle("c"), type="Var", source="@lib/core/operators.nom[4614:4616]"}, type= "Action", source= "@lib/core/operators.nom[4608:4616]", stub= ""}, "->", SyntaxTree{SyntaxTree{SyntaxTree{mangle("a"), type="Var", source="@lib/core/operators.nom[4623:4625]"}, "<", SyntaxTree{mangle("b"), type="Var", source="@lib/core/operators.nom[4628:4630]"}, type= "Action", source= "@lib/core/operators.nom[4623:4630]", stub= "1 <"}, "and", SyntaxTree{SyntaxTree{mangle("b"), type="Var", source="@lib/core/operators.nom[4637:4639]"}, "<=", SyntaxTree{mangle("c"), type="Var", source="@lib/core/operators.nom[4643:4645]"}, type= "Action", source= "@lib/core/operators.nom[4637:4645]", stub= "1 <="}, type= "Action", source= "@lib/core/operators.nom[4622:4646]", stub= "1 and"}, type= "Action", source= "@lib/core/operators.nom[4607:4647]", stub= "1 ->"}, x, y, z, type= "Action", source= "@lib/core/operators.nom[4606:4657]", stub= ""}
end);
COMPILE_RULES["1 <= 2 <="] = (function(_ENV, _tree, x, y, z)
    local mangle = mangler()
    return SyntaxTree{SyntaxTree{SyntaxTree{SyntaxTree{mangle("a"), type="Var", source="@lib/core/operators.nom[4689:4691]"}, SyntaxTree{mangle("b"), type="Var", source="@lib/core/operators.nom[4692:4694]"}, SyntaxTree{mangle("c"), type="Var", source="@lib/core/operators.nom[4695:4697]"}, type= "Action", source= "@lib/core/operators.nom[4689:4697]", stub= ""}, "->", SyntaxTree{SyntaxTree{SyntaxTree{mangle("a"), type="Var", source="@lib/core/operators.nom[4704:4706]"}, "<=", SyntaxTree{mangle("b"), type="Var", source="@lib/core/operators.nom[4710:4712]"}, type= "Action", source= "@lib/core/operators.nom[4704:4712]", stub= "1 <="}, "and", SyntaxTree{SyntaxTree{mangle("b"), type="Var", source="@lib/core/operators.nom[4719:4721]"}, "<=", SyntaxTree{mangle("c"), type="Var", source="@lib/core/operators.nom[4725:4727]"}, type= "Action", source= "@lib/core/operators.nom[4719:4727]", stub= "1 <="}, type= "Action", source= "@lib/core/operators.nom[4703:4728]", stub= "1 and"}, type= "Action", source= "@lib/core/operators.nom[4688:4729]", stub= "1 ->"}, x, y, z, type= "Action", source= "@lib/core/operators.nom[4687:4739]", stub= ""}
end);
COMPILE_RULES["1 > 2 >"] = (function(_ENV, _tree, x, y, z)
    local mangle = mangler()
    return SyntaxTree{SyntaxTree{SyntaxTree{SyntaxTree{mangle("a"), type="Var", source="@lib/core/operators.nom[4769:4771]"}, SyntaxTree{mangle("b"), type="Var", source="@lib/core/operators.nom[4772:4774]"}, SyntaxTree{mangle("c"), type="Var", source="@lib/core/operators.nom[4775:4777]"}, type= "Action", source= "@lib/core/operators.nom[4769:4777]", stub= ""}, "->", SyntaxTree{SyntaxTree{SyntaxTree{mangle("a"), type="Var", source="@lib/core/operators.nom[4784:4786]"}, ">", SyntaxTree{mangle("b"), type="Var", source="@lib/core/operators.nom[4789:4791]"}, type= "Action", source= "@lib/core/operators.nom[4784:4791]", stub= "1 >"}, "and", SyntaxTree{SyntaxTree{mangle("b"), type="Var", source="@lib/core/operators.nom[4798:4800]"}, ">", SyntaxTree{mangle("c"), type="Var", source="@lib/core/operators.nom[4803:4805]"}, type= "Action", source= "@lib/core/operators.nom[4798:4805]", stub= "1 >"}, type= "Action", source= "@lib/core/operators.nom[4783:4806]", stub= "1 and"}, type= "Action", source= "@lib/core/operators.nom[4768:4807]", stub= "1 ->"}, x, y, z, type= "Action", source= "@lib/core/operators.nom[4767:4817]", stub= ""}
end);
COMPILE_RULES["1 >= 2 >"] = (function(_ENV, _tree, x, y, z)
    local mangle = mangler()
    return SyntaxTree{SyntaxTree{SyntaxTree{SyntaxTree{mangle("a"), type="Var", source="@lib/core/operators.nom[4848:4850]"}, SyntaxTree{mangle("b"), type="Var", source="@lib/core/operators.nom[4851:4853]"}, SyntaxTree{mangle("c"), type="Var", source="@lib/core/operators.nom[4854:4856]"}, type= "Action", source= "@lib/core/operators.nom[4848:4856]", stub= ""}, "->", SyntaxTree{SyntaxTree{SyntaxTree{mangle("a"), type="Var", source="@lib/core/operators.nom[4863:4865]"}, ">=", SyntaxTree{mangle("b"), type="Var", source="@lib/core/operators.nom[4869:4871]"}, type= "Action", source= "@lib/core/operators.nom[4863:4871]", stub= "1 >="}, "and", SyntaxTree{SyntaxTree{mangle("b"), type="Var", source="@lib/core/operators.nom[4878:4880]"}, ">", SyntaxTree{mangle("c"), type="Var", source="@lib/core/operators.nom[4883:4885]"}, type= "Action", source= "@lib/core/operators.nom[4878:4885]", stub= "1 >"}, type= "Action", source= "@lib/core/operators.nom[4862:4886]", stub= "1 and"}, type= "Action", source= "@lib/core/operators.nom[4847:4887]", stub= "1 ->"}, x, y, z, type= "Action", source= "@lib/core/operators.nom[4846:4897]", stub= ""}
end);
COMPILE_RULES["1 > 2 >="] = (function(_ENV, _tree, x, y, z)
    local mangle = mangler()
    return SyntaxTree{SyntaxTree{SyntaxTree{SyntaxTree{mangle("a"), type="Var", source="@lib/core/operators.nom[4928:4930]"}, SyntaxTree{mangle("b"), type="Var", source="@lib/core/operators.nom[4931:4933]"}, SyntaxTree{mangle("c"), type="Var", source="@lib/core/operators.nom[4934:4936]"}, type= "Action", source= "@lib/core/operators.nom[4928:4936]", stub= ""}, "->", SyntaxTree{SyntaxTree{SyntaxTree{mangle("a"), type="Var", source="@lib/core/operators.nom[4943:4945]"}, ">", SyntaxTree{mangle("b"), type="Var", source="@lib/core/operators.nom[4948:4950]"}, type= "Action", source= "@lib/core/operators.nom[4943:4950]", stub= "1 >"}, "and", SyntaxTree{SyntaxTree{mangle("b"), type="Var", source="@lib/core/operators.nom[4957:4959]"}, ">=", SyntaxTree{mangle("c"), type="Var", source="@lib/core/operators.nom[4963:4965]"}, type= "Action", source= "@lib/core/operators.nom[4957:4965]", stub= "1 >="}, type= "Action", source= "@lib/core/operators.nom[4942:4966]", stub= "1 and"}, type= "Action", source= "@lib/core/operators.nom[4927:4967]", stub= "1 ->"}, x, y, z, type= "Action", source= "@lib/core/operators.nom[4926:4977]", stub= ""}
end);
COMPILE_RULES["1 >= 2 >="] = (function(_ENV, _tree, x, y, z)
    local mangle = mangler()
    return SyntaxTree{SyntaxTree{SyntaxTree{SyntaxTree{mangle("a"), type="Var", source="@lib/core/operators.nom[5009:5011]"}, SyntaxTree{mangle("b"), type="Var", source="@lib/core/operators.nom[5012:5014]"}, SyntaxTree{mangle("c"), type="Var", source="@lib/core/operators.nom[5015:5017]"}, type= "Action", source= "@lib/core/operators.nom[5009:5017]", stub= ""}, "->", SyntaxTree{SyntaxTree{SyntaxTree{mangle("a"), type="Var", source="@lib/core/operators.nom[5024:5026]"}, ">=", SyntaxTree{mangle("b"), type="Var", source="@lib/core/operators.nom[5030:5032]"}, type= "Action", source= "@lib/core/operators.nom[5024:5032]", stub= "1 >="}, "and", SyntaxTree{SyntaxTree{mangle("b"), type="Var", source="@lib/core/operators.nom[5039:5041]"}, ">=", SyntaxTree{mangle("c"), type="Var", source="@lib/core/operators.nom[5045:5047]"}, type= "Action", source= "@lib/core/operators.nom[5039:5047]", stub= "1 >="}, type= "Action", source= "@lib/core/operators.nom[5023:5048]", stub= "1 and"}, type= "Action", source= "@lib/core/operators.nom[5008:5049]", stub= "1 ->"}, x, y, z, type= "Action", source= "@lib/core/operators.nom[5007:5059]", stub= ""}
end);
--  TODO: optimize for common case where x,y,z are all either variables or number literals;
--  Boolean Operators;
TESTS["@lib/core/operators.nom[5179:5331]"] = "\n(barfer) means (fail \"short circuiting failed\")\nassume (((no) and (barfer)) == (no))\nassume ((no) or (yes))\nassume ((yes) or (barfer))";
COMPILE_RULES["1 and"] = (function(_ENV, _tree, x, y)
    return LuaCode:from("@lib/core/operators.nom[5356:5399]", "(", _ENV:_1_as_lua_expr(x),
        " and ", _ENV:_1_as_lua_expr(y), ")")
end);
COMPILE_RULES["1 or"] = (function(_ENV, _tree, x, y)
    return LuaCode:from("@lib/core/operators.nom[5423:5465]", "(", _ENV:_1_as_lua_expr(x),
        " or ", _ENV:_1_as_lua_expr(y), ")")
end);

-- File lib/core/operators.nom chunk #5
--  Bitwise Operators;
--  These can break if running the precompiled code from another Lua version:;
if "LuaJIT 2.0.5" ~= LUA_VERSION then
    at_1_fail("@lib/core/operators.nom[5744:5959]",
    Text("This code was precompiled with ", "LuaJIT 2.0.5",
  ", but it is being run with ", LUA_VERSION,
  ". This will cause problems with bitwise operations.")
)
end
--  TODO: implement OR, XOR, AND for multiple operands?;
TESTS["@lib/core/operators.nom[6042:6199]"] = "\nassume (~(~5) == 5)\nassume ((1 | 4) == 5)\nassume ((1 ~ 3) == 2)\nassume ((1 & 3) == 1)\nassume ((1 << 2) == 4)\nassume ((4 >> 2) == 1)";
--  Lua 5.3 introduced bit operators like | and &. Use them when possible, otherwise
-- fall back to bit.bor(), bit.band(), etc.;
if (jit or (LUA_API == "Lua 5.2")) then;
COMPILE_RULES["NOT"] = (function(_ENV, _tree, _)
    return LuaCode:from("@lib/core/operators.nom[6414:6442]", "Bit.bnot(",
        _ENV:_1_as_lua_expr(_), ")")
end)
COMPILE_RULES["~"] = COMPILE_RULES["NOT"];
COMPILE_RULES["1 OR"] = (function(_ENV, _tree, x, y)
    return LuaCode:from("@lib/core/operators.nom[6482:6529]", "Bit.bor(",
        _ENV:_1_as_lua_expr(x), ", ", _ENV:_1_as_lua_expr(y), ")")
end)
COMPILE_RULES["1 |"] = COMPILE_RULES["1 OR"];
COMPILE_RULES["1 XOR"] = (function(_ENV, _tree, x, y)
    return LuaCode:from("@lib/core/operators.nom[6571:6619]", "Bit.bxor(",
        _ENV:_1_as_lua_expr(x), ", ", _ENV:_1_as_lua_expr(y), ")")
end)
COMPILE_RULES["1 ~"] = COMPILE_RULES["1 XOR"];
COMPILE_RULES["1 AND"] = (function(_ENV, _tree, x, y)
    return LuaCode:from("@lib/core/operators.nom[6661:6709]", "Bit.band(",
        _ENV:_1_as_lua_expr(x), ", ", _ENV:_1_as_lua_expr(y), ")")
end)
COMPILE_RULES["1 &"] = COMPILE_RULES["1 AND"];
COMPILE_RULES["1 LSHIFT"] = (function(_ENV, _tree, x, shift)
    return LuaCode:from("@lib/core/operators.nom[6763:6817]", "Bit.lshift(",
        _ENV:_1_as_lua_expr(x), ", ", _ENV:_1_as_lua_expr(shift), ")")
end)
COMPILE_RULES["1 <<"] = COMPILE_RULES["1 LSHIFT"];
COMPILE_RULES["1 RSHIFT"] = (function(_ENV, _tree, x, shift)
    return LuaCode:from("@lib/core/operators.nom[6871:6925]", "Bit.rshift(",
        _ENV:_1_as_lua_expr(x), ", ", _ENV:_1_as_lua_expr(shift), ")")
end)
COMPILE_RULES["1 >>"] = COMPILE_RULES["1 RSHIFT"];
else;
COMPILE_RULES["NOT"] = (function(_ENV, _tree, _)
    return LuaCode:from("@lib/core/operators.nom[6966:6989]", "(~(", _ENV:_1_as_lua_expr(_),
        "))")
end)
COMPILE_RULES["~"] = COMPILE_RULES["NOT"];
COMPILE_RULES["1 OR"] = (function(_ENV, _tree, x, y)
    return LuaCode:from("@lib/core/operators.nom[7025:7066]", "(", _ENV:_1_as_lua_expr(x),
        " | ", _ENV:_1_as_lua_expr(y), ")")
end)
COMPILE_RULES["1 |"] = COMPILE_RULES["1 OR"];
COMPILE_RULES["1 XOR"] = (function(_ENV, _tree, x, y)
    return LuaCode:from("@lib/core/operators.nom[7103:7144]", "(", _ENV:_1_as_lua_expr(x),
        " ~ ", _ENV:_1_as_lua_expr(y), ")")
end)
COMPILE_RULES["1 ~"] = COMPILE_RULES["1 XOR"];
COMPILE_RULES["1 AND"] = (function(_ENV, _tree, x, y)
    return LuaCode:from("@lib/core/operators.nom[7181:7222]", "(", _ENV:_1_as_lua_expr(x),
        " & ", _ENV:_1_as_lua_expr(y), ")")
end)
COMPILE_RULES["1 &"] = COMPILE_RULES["1 AND"];
COMPILE_RULES["1 LSHIFT"] = (function(_ENV, _tree, x, shift)
    return LuaCode:from("@lib/core/operators.nom[7275:7321]", "(", _ENV:_1_as_lua_expr(x),
        " << ", _ENV:_1_as_lua_expr(shift), ")")
end)
COMPILE_RULES["1 <<"] = COMPILE_RULES["1 LSHIFT"];
COMPILE_RULES["1 RSHIFT"] = (function(_ENV, _tree, x, shift)
    return LuaCode:from("@lib/core/operators.nom[7375:7421]", "(", _ENV:_1_as_lua_expr(x),
        " >> ", _ENV:_1_as_lua_expr(shift), ")")
end)
COMPILE_RULES["1 >>"] = COMPILE_RULES["1 RSHIFT"];
end;
--  Unary operators;
TESTS["@lib/core/operators.nom[7459:7518]"] = "\nassume ((- 5) == -5)\nassume ((not (yes)) == (no))";
COMPILE_RULES["-"] = (function(_ENV, _tree, _)
    return LuaCode:from("@lib/core/operators.nom[7534:7557]", "(-(", _ENV:_1_as_lua_expr(_),
        "))")
end);
COMPILE_RULES["not"] = (function(_ENV, _tree, _)
    return LuaCode:from("@lib/core/operators.nom[7578:7602]", "(not ", _ENV:_1_as_lua_expr(_),
        ")")
end);
TESTS["@lib/core/operators.nom[7607:7675]"] = "\nassume ((size of [1, 2, 3]) == 3)\nassume (#[1, 2, 3] == 3)";
--  Length;
COMPILE_RULES["#"] = (function(_ENV, _tree, list)
    local list_lua = _ENV:_1_as_lua_expr(list)
    if list_lua:text() == "..." then
        return LuaCode("select('#', ...)")
    end
    return LuaCode("(#", list_lua, ")");
end)
COMPILE_RULES["size of"] = COMPILE_RULES["#"];
COMPILE_RULES["1 is empty"] = (function(_ENV, _tree, list)
    local mangle = mangler()
    return SyntaxTree{SyntaxTree{"#", list, type= "Action", source= "@lib/core/operators.nom[7965:7971]", stub= "#"}, "==", SyntaxTree{"0", type= "Number", source= "@lib/core/operators.nom[7975:7976]"}, type= "Action", source= "@lib/core/operators.nom[7965:7976]", stub= "1 =="}
end);

-- File lib/core/operators.nom chunk #6
--  Update operators;
TESTS["@lib/core/operators.nom[8086:8293]"] = "\n$x = 1\n$x += 1\nunless ($x == 2):\n    fail \"+= failed\"\n$x *= 2\nunless ($x == 4):\n    fail \"*= failed\"\nwrap $x around 3\nunless ($x == 1):\n    fail \"wrap around failed\"";
COMPILE_RULES["1 +="] = (function(_ENV, _tree, var, _)
    local mangle = mangler()
    return SyntaxTree{var, "=", SyntaxTree{SyntaxTree{var, "or", SyntaxTree{"0", type= "Number", source= "@lib/core/operators.nom[8335:8336]"}, type= "Action", source= "@lib/core/operators.nom[8327:8336]", stub= "1 or"}, "+", _, type= "Action", source= "@lib/core/operators.nom[8326:8341]", stub= "1 +"}, type= "Action", source= "@lib/core/operators.nom[8318:8342]", stub= "1 ="}
end);
COMPILE_RULES["1 -="] = (function(_ENV, _tree, var, _)
    local mangle = mangler()
    return SyntaxTree{var, "=", SyntaxTree{SyntaxTree{var, "or", SyntaxTree{"0", type= "Number", source= "@lib/core/operators.nom[8384:8385]"}, type= "Action", source= "@lib/core/operators.nom[8376:8385]", stub= "1 or"}, "-", _, type= "Action", source= "@lib/core/operators.nom[8375:8390]", stub= "1 -"}, type= "Action", source= "@lib/core/operators.nom[8367:8391]", stub= "1 ="}
end);
COMPILE_RULES["1 *="] = (function(_ENV, _tree, var, _)
    local mangle = mangler()
    return SyntaxTree{var, "=", SyntaxTree{SyntaxTree{var, "or", SyntaxTree{"1", type= "Number", source= "@lib/core/operators.nom[8433:8434]"}, type= "Action", source= "@lib/core/operators.nom[8425:8434]", stub= "1 or"}, "*", _, type= "Action", source= "@lib/core/operators.nom[8424:8439]", stub= "1 *"}, type= "Action", source= "@lib/core/operators.nom[8416:8440]", stub= "1 ="}
end);
COMPILE_RULES["1 /="] = (function(_ENV, _tree, var, _)
    local mangle = mangler()
    return SyntaxTree{var, "=", SyntaxTree{var, "/", _, type= "Action", source= "@lib/core/operators.nom[8473:8481]", stub= "1 /"}, type= "Action", source= "@lib/core/operators.nom[8465:8482]", stub= "1 ="}
end);
COMPILE_RULES["1 ^="] = (function(_ENV, _tree, var, _)
    local mangle = mangler()
    return SyntaxTree{var, "=", SyntaxTree{var, "^", _, type= "Action", source= "@lib/core/operators.nom[8515:8523]", stub= "1 ^"}, type= "Action", source= "@lib/core/operators.nom[8507:8524]", stub= "1 ="}
end);
COMPILE_RULES["1 and ="] = (function(_ENV, _tree, var, _)
    local mangle = mangler()
    return SyntaxTree{var, "=", SyntaxTree{var, "and", _, type= "Action", source= "@lib/core/operators.nom[8559:8569]", stub= "1 and"}, type= "Action", source= "@lib/core/operators.nom[8551:8570]", stub= "1 ="}
end);
COMPILE_RULES["1 or ="] = (function(_ENV, _tree, var, _)
    local mangle = mangler()
    return SyntaxTree{var, "=", SyntaxTree{var, "or", _, type= "Action", source= "@lib/core/operators.nom[8604:8613]", stub= "1 or"}, type= "Action", source= "@lib/core/operators.nom[8596:8614]", stub= "1 ="}
end);
COMPILE_RULES["wrap 1 around"] = (function(_ENV, _tree, var, _)
    local mangle = mangler()
    return SyntaxTree{var, "=", SyntaxTree{var, "wrapped", "around", _, type= "Action", source= "@lib/core/operators.nom[8656:8677]", stub= "1 wrapped around"}, type= "Action", source= "@lib/core/operators.nom[8648:8678]", stub= "1 ="}
end);
