#!/usr/bin/env nomsu -V7.0.0-- File lib/tools/find.nom chunk #1
-- 
-- This is a tool to find syntax trees matching a pattern. "*" is a wildcard
-- that will match any subtree, and "**" is a wildcard that will match any
-- 0 or more subtrees. "**" is greedy, so extra arguments after it will
-- not match.
-- 
-- nomsu -t find [flags] "* squared" file1 file2...
-- 
-- Flags:
-- -l  List only the names of the files with matches
-- --wildcard=<wildcard>  Specify a custom wildcard (in case you need to
-- match an action with a "*" in the name)
-- 
-- Output:
-- <filename>:<line number>:
-- <matching lines>
-- ;
_ENV:use("filesystem");
_ENV:use("consolecolor");
_ENV:use("commandline");

-- File lib/tools/find.nom chunk #2
run_with = (function(args)
    local wildcard, pattern, patternx5Ftree, _1_matches, filenames, file, code, tree, msg, results, linex5Fnum, keysx78036;
    wildcard = (args.wildcard or "%*");
    pattern = args.extras[1];
    if any_of(a_List{(not pattern), (pattern == "*"), (pattern == "**")}) then
        usage(
        Text("nomsu -t find [-l] [--wildcard=<wildcard>] <pattern>, where <pattern> is valid Nomsu code"));
    end
    pattern = pattern:with_1_x2Dx3E(Text(wildcard, wildcard), "$multi_wildcard");
    pattern = pattern:with_1_x2Dx3E(wildcard, "$wildcard");
    patternx5Ftree = _1_parsed(pattern);
    _1_matches = (function(tree, patt)
        if (not SyntaxTree:is_instance(tree)) then
            do return (false) end
        elseif ((patt.type == "Var") and (patt[1] == "wildcard")) then
            do return (true) end
        elseif (tree.type ~= patt.type) then
            do return (false) end
        elseif (tree.type == "Action") then
            if (tree:get_stub() ~= patt:get_stub()) then
                do return (false) end
            end
        end --when;
        do -- for-loop
            local _start = 1
        for _=_start,(#patt) do
                if SyntaxTree:is_instance(patt[_]) then
            if (patt[_] == SyntaxTree{type="Var", source=Source("lib/tools/find.nom", 1660, 1675), "multi_wildcard"}) then
            do return (true) end
        end
        if (not _1_matches(tree[_], patt[_])) then
            do return (false) end
        end
        else
            if (not (tree[_] == patt[_])) then
            do return (false) end
        end
        end
            end
        end -- for-loop;
        if ((#tree) ~= (#patt)) then
            do return (false) end
        end
        do return (true) end
    end);
    filenames = args.extras:from_1_to(2, -1);
    if ((#filenames) == 0) then
        say(
        Text("Warning: searching stdin (ctrl-d to abort). To avoid this message, use nomsu -t find -"));
    filenames = a_List{"stdin"};
    end
    do -- for-loop
        for _i,filename in _ipairs(filenames) do
            file = read_file(filename);
    if (not file) then
        at_1_fail("@lib/tools/find.nom[2263:2301]",
        Text("File does not exist: ", filename)
    );
    end
    code = NomsuCode_from(Source(filename, 1, (#file)), file);
    do
        local _fell_through = false
        local _result = {xpcall(function()
            tree = _1_parsed(code);
            _fell_through = true
        end, enhance_error)}
        if _result[1] then
            -- Success:
    if not _fell_through then return table.unpack(_result, 2) end
        else
            -- Failure:
    
    msg = _result[2]
    say(red(Text(filename, " failed to parse:\n", msg)));
    tree = (nil);
        end
    end
    if (not tree) then
        goto continue_filename;
    end
    results = a_List{};
    do
        local _stack_t = a_List{tree}
        while #_stack_t > 0 do
            t = table.remove(_stack_t, 1)
            if _1_matches(t, patternx5Ftree) then
        linex5Fnum = file:line_number_at(t.source.start);
    results:add(
        a_Dict{line = linex5Fnum; text = Text(blue(Text(filename, ":", linex5Fnum, ":")), "\n", source_lines_of(t));});
    end
    do -- for-loop
        for _i,sub in _ipairs(t) do
            if SyntaxTree:is_instance(sub) then
        table.insert(_stack_t, sub);
    end
        end
    end -- for-loop;
        end -- Recursive loop
    end -- Recursive scope;
    if args.l then
        if ((#results) > 0) then
        say(filename);
    end
    else
        do
        keysx78036 = (function(d)
        local mt = {}
        for k,v in pairs(getmetatable(d) or {}) do mt[k] = v end
        mt.__index = function(self, _)
            local value = _.line
            self[_] = value
            return value
        end
        return setmetatable(d, mt)
    end)(a_Dict{});
    table.sort(results, function(x,y) return keysx78036[x] < keysx78036[y] end);
    end -- do;
    do -- for-loop
        for _i,_ in _ipairs(results) do
            say(_.text);
        end
    end -- for-loop;
    end
            ::continue_filename::
        end
    end -- for-loop;
end);
if WAS_RUN_DIRECTLY then
    run_with(COMMAND_LINE_ARGS);
end
