#!/usr/bin/env nomsu -V7.0.0-- File lib/tools/repl.nom chunk #1
-- 
-- This file defines a Read-Evaluate-Print-Loop (REPL) for Nomsu
-- ;
_ENV:use("consolecolor");
_ENV:use("filesystem");
_ENV:use("commandline");

-- File lib/tools/repl.nom chunk #2
help = (function()
    say(
        Text("This is the Nomsu v", NOMSU_VERSION:joined_with("."),
          " interactive console.\n",
          "You can type in Nomsu code here and hit 'enter' twice to run it.\n",
          "To exit, type 'exit' or 'quit' and hit enter twice."));
end);
tutorial = (function()
    (_ENV:use("tools/tutorial").run_with)(a_Dict{extras = a_List{};});
    exit();
end);
run_with = (function(args)
    local say_results_of, buff, line, tree, err, lua;
    say(
        Text("\n", bright(), underscore(), "Welcome to the Nomsu v",
          NOMSU_VERSION:joined_with("."), " interactive console!", reset_color(), "\n",
          "       press 'enter' twice to run a command\n",
          "       type 'tutorial' to run the tutorial\n"));
    --  Best way I know of to detect the number of return values and only
    -- print if it's >0:;
    say_results_of = (function(...)
        local N, ret;
        N = select("#", ...);
        if (N == 0) then
            do return  end
        end
        do -- for-loop
            local _start = 1
        for _=_start,N do
                ret = select(_, ...);
        if _1_is(ret, "Text") then
            ret = quote(ret);
        end
        say(Text(ret));
            end
        end -- for-loop;
    end);
    while (true) do
        say_1_inline(bright(yellow(">> ")));
    buff = a_List{};
    while (true) do
        say_1_inline(bright());
    line = io.read("*L");
    say_1_inline(reset_color());
    if ((line == "\n") or (not line)) then
        if ((#buff) > 0) then
        --  clear the line;
    if COLOR_ENABLED then
        say_1_inline("\027[1A\027[2K");
    end
    end
    goto label_run_buffer;
    end
    buff:add(line:with_1_x2Dx3E("\009", "    "));
    say_1_inline(dim(yellow(".. ")));
    end --while-loop;
    ::label_run_buffer::;
    if ((#buff) == 0) then
        break;
    end
    buff = buff:joined();
    spoof_file(buff);
    do
        local _fell_through = false
        local _result = {xpcall(function()
            tree = _1_parsed(buff);
            _fell_through = true
        end, enhance_error)}
        if _result[1] then
            -- Success:
    if not _fell_through then return table.unpack(_result, 2) end
        else
            -- Failure:
    
    err = _result[2]
    say(err);
    goto continue;
        end
    end
    if (not tree) then
        goto continue;
    end
    if (tree.type == "Comment") then
        say(dim(Text("Comment:", tree[1])));
    goto continue;
    end
    if (tree.type ~= "FileChunks") then
        tree = a_List{tree};
    end
    do -- for-loop
        for _i,chunk in _ipairs(tree) do
            do
        local _fell_through = false
        local _result = {xpcall(function()
            lua = _ENV:compile(chunk);
            _fell_through = true
        end, enhance_error)}
        if _result[1] then
            -- Success:
    if not _fell_through then return table.unpack(_result, 2) end
        else
            -- Failure:
    
    err = _result[2]
    say(err);
        end
    end
    if (not lua) then
        goto continue;
    end
    --  TODO: this is a bit hacky, it just defaults variables to global
    -- so that stuff mostly works across multiple lines. It would be
    -- nicer if local variables actually worked.;
    lua:remove_free_vars();
    if load(Text("return ", lua:text())) then
        lua:prepend("return ");
    end
    do
        local _fell_through = false
        local _result = {xpcall(function()
            say_results_of(_ENV:run(lua));
            _fell_through = true
        end, enhance_error)}
        if _result[1] then
            -- Success:
    if not _fell_through then return table.unpack(_result, 2) end
        else
            -- Failure:
    
    err = _result[2]
    say(err);
        end
    end
            ::continue::
        end
    end -- for-loop;
        ::continue::
    end --while-loop;
end);
if WAS_RUN_DIRECTLY then
    run_with(COMMAND_LINE_ARGS);
end
