#!/usr/bin/env nomsu -V7.0.0-- File lib/tools/replace.nom chunk #1
-- 
-- This is a tool to replace syntax trees with something new.
-- 
-- Usage:
--     nomsu -t replace [-i] [-f] [-q] [--literal="$v1 $v2..."] <pattern> <replacement> file1 file2...
-- 
-- Example:
--     nomsu -t replace "($1 and $2) and $3" "all of [$1, $2, $3]" my_file.nom
-- 
-- If the "-i" flag is used, the file(s) will be edited in-place.
-- When editing in-place, if the "-f" flag is not used, each change will be
--     run past the user first.
-- If the "-q" flag is used and a file fails to parse, the original file
--     contents will be output.
-- If no files are passed in, this will read from stdin.
-- ;
_ENV:use("filesystem");
_ENV:use("consolecolor");
_ENV:use("commandline");

-- File lib/tools/replace.nom chunk #2
run_with = (function(args)
    local pattern, replacement, patternx5Ftree, replacementx5Ftree, literalx5Fvars, patternx5Fvars, _1_matches_2_with, filenames, file, code, tree, msg, replaced, matched, userx5Fanswers, _1_with_replacements, tree2;
    if ((#args.extras) < 2) then
        at_1_fail("@lib/tools/replace.nom[873:991]",
        Text("Usage: nomsu -t replace [--literal=\"$v1 $v2...\"] <pattern> <replacement> file1 file2...")
    );
    end
    pattern = args.extras[1];
    replacement = args.extras[2];
    patternx5Ftree = _1_parsed(pattern);
    replacementx5Ftree = _1_parsed(replacement);
    literalx5Fvars = a_Dict{};
    if args.literal then
        do -- for-loop
        for _i,var in _ipairs(args.literal:all_matches_of("$([^ ]*)")) do
            literalx5Fvars[var] = (true);
        end
    end -- for-loop;
    end
    if ((patternx5Ftree.type == "Var") and (not literalx5Fvars[patternx5Ftree[1]])) then
        at_1_fail("@lib/tools/replace.nom[1383:1429]",
        "Pattern matches every part of the file."
    );
    end
    patternx5Fvars = a_Dict(function(add, add_1_x3D)
        do
            local _stack__ = a_List{patternx5Ftree}
            while #_stack__ > 0 do
                _ = table.remove(_stack__, 1)
                if ((_.type == "Var") and (not literalx5Fvars[_[1]])) then
            add(_[1]);
        end
        do -- for-loop
            for _i,child in _ipairs(_) do
                if _1_is(child, "a Syntax Tree") then
            table.insert(_stack__, child);
        end
            end
        end -- for-loop;
            end -- Recursive loop
        end -- Recursive scope;
    end);
    --  TODO: support wildcards and unpacking
    -- e.g. nomsu -t replace "test(: $test; *$more_tests)" "*$more_tests; *$test";
    _1_matches_2_with = (function(tree, patt, substitutionx5Fvalues)
        local newx5Fvalues;
        --  TODO: optimize;
        substitutionx5Fvalues = a_Dict(function(add, add_1_x3D)
            do -- for-loop
                for k,v in pairs(substitutionx5Fvalues) do
                    add_1_x3D(k, v);
                end
            end -- for-loop;
        end);
        if (not SyntaxTree:is_instance(tree)) then
            do return (false) end
        elseif ((patt.type == "Var") and patternx5Fvars[patt[1]]) then
            if substitutionx5Fvalues[patt[1]] then
                if (tree == substitutionx5Fvalues[patt[1]]) then
                do return substitutionx5Fvalues end
            else
                do return (nil) end
            end
            else
                substitutionx5Fvalues[patt[1]] = tree;
            do return substitutionx5Fvalues end
            end
        elseif (tree.type ~= patt.type) then
            do return (nil) end
        elseif (tree.type == "Action") then
            if (tree:get_stub() ~= patt:get_stub()) then
                do return (nil) end
            end
        end --when;
        do -- for-loop
            local _start = 1
        for _=_start,(#patt) do
                if SyntaxTree:is_instance(patt[_]) then
            newx5Fvalues = _1_matches_2_with(tree[_], patt[_], substitutionx5Fvalues);
        if (not newx5Fvalues) then
            do return (nil) end
        end
        do -- for-loop
            for k,v in pairs(newx5Fvalues) do
                substitutionx5Fvalues[k] = v;
            end
        end -- for-loop;
        else
            if (not (tree[_] == patt[_])) then
            do return (nil) end
        end
        end
            end
        end -- for-loop;
        if ((#tree) ~= (#patt)) then
            do return (nil) end
        end
        do return substitutionx5Fvalues end
    end);
    filenames = args.extras:from_1_to(3, -1);
    if ((#filenames) == 0) then
        say(
        Text("Warning: searching stdin (ctrl-d to abort). To avoid this message, use nomsu -t find -"));
    filenames = a_List{"stdin"};
    end
    do -- for-loop
        for _i,filename in _ipairs(filenames) do
            file = read_file(filename);
    if (not file) then
        at_1_fail("@lib/tools/replace.nom[3551:3589]",
        Text("File does not exist: ", filename)
    );
    end
    code = NomsuCode_from(Source(filename, 1, (#file)), file);
    do
        local _fell_through = false
        local _result = {xpcall(function()
            tree = _1_parsed(code);
            _fell_through = true
        end, enhance_error)}
        if _result[1] then
            -- Success:
    if not _fell_through then return table.unpack(_result, 2) end
        else
            -- Failure:
    
    msg = _result[2]
    if args.q then
        if (not args.i) then
        say(code);
    end
    else
        say(msg);
    end
        end
    end
    if (not tree) then
        goto continue_filename;
    end
    replaced = a_Dict{};
    matched = a_Dict{};
    userx5Fanswers = a_Dict{};
    _1_with_replacements = (function(tree)
        return tree:with(
            (function(t)
                local values, ret;
                values = _1_matches_2_with(t, patternx5Ftree, a_Dict{});
                if values then
                    matched[t] = (true);
                do -- for-loop
                    for k,v in pairs(values) do
                        values[k] = _1_with_replacements(v);
                    end
                end -- for-loop;
                ret = replacementx5Ftree:with(values);
                if (args.i and (not args.f)) then
                    if (userx5Fanswers[t] == (nil)) then
                    if ((#userx5Fanswers) > 0) then
                    say("");
                end
                userx5Fanswers[t] = "n";
                say(Text(bright(), "Should this:"));
                say(
                    Text("    ", bright(), yellow(),
                      (Text(_1_as_nomsu(_1_with_replacements(t)))):with_1_x2Dx3E("\n", "\n    "), "",
                      reset_color()));
                say(Text(bright(), "..be replaced with:"));
                say(
                    Text("    ", bright(), blue(),
                      (Text(_1_as_nomsu(ret))):with_1_x2Dx3E("\n", "\n    "), reset_color()));
                userx5Fanswers[t] = ask(Text(bright(), "..? [Y/n]", reset_color(), " "));
                end
                if (userx5Fanswers[t] == "n") then
                    do return (nil) end
                end
                end
                replaced[t] = (true);
                do return ret end
                end
            end))
    end);
    tree2 = _1_with_replacements(tree);
    if args.i then
        if ((#userx5Fanswers) > 0) then
        say("");
    end
    say(
        Text((#replaced), "/", (#matched), " replacement",
          (((#replaced) == 1) and "" or "s"), " in ", filename));
    if ((#replaced) > 0) then
        write_1_to_file(Text(_1_as_nomsu(tree2)), filename);
    end
    else
        say(_1_as_nomsu(tree2));
    end
            ::continue_filename::
        end
    end -- for-loop;
end);
if WAS_RUN_DIRECTLY then
    run_with(COMMAND_LINE_ARGS);
end
