#!/usr/bin/env nomsu -V7.0.0-- File lib/tools/test.nom chunk #1
-- 
-- Tool to run all tests in a file (i.e. the code block inside a call to 'test $'). Usage:
-- nomsu tools/test.nom file1 file2 directory1 ...
-- ;
_ENV:use("filesystem");
_ENV:use("consolecolor");
_ENV:use("commandline");

-- File lib/tools/test.nom chunk #2
run_with = (function(args)
    local file, test_environment, version, filex5Ftests, keysx78034, failures, msg, src, l1, l2;
    do -- for-loop
        for i,filename in pairs(args.extras) do
            file = read_file(filename);
    if (not file) then
        at_1_fail("@lib/tools/test.nom[467:498]",
        Text("Couldn't find ", filename)
    );
    end
    test_environment = new_environment();
    test_environment:export(filename);
    version = file:matching(Text("#![^\n", "]* nomsu %-V[ ]*([%d.]*)"));
    filex5Ftests = a_List{};
    do -- for-loop
        for src,test in pairs(_ENV:Module(filename).TESTS) do
            if version then
        test = Text("#!/usr/bin/env nomsu -V", version, "\n", test);
    end
    filex5Ftests:add(a_Dict{test = test; source = src;});
        end
    end -- for-loop;
    if (not ((#filex5Ftests) == 0)) then
        do
        keysx78034 = (function(d)
        local mt = {}
        for k,v in pairs(getmetatable(d) or {}) do mt[k] = v end
        mt.__index = function(self, _)
            local value = _.source
            self[_] = value
            return value
        end
        return setmetatable(d, mt)
    end)(a_Dict{});
    table.sort(filex5Ftests, function(x,y) return keysx78034[x] < keysx78034[y] end);
    end -- do;
    say_1_inline(Text("[ .. ] ", filename));
    io.flush();
    if args.v then
        say("");
    end
    failures = a_List{};
    do -- for-loop
        for _i,_ in _ipairs(filex5Ftests) do
            if args.v then
        say(Text("    ", yellow(_.test:with_1_x2Dx3E("\n", "\n    "))));
    end
    if args.e then
        test_environment:run(_.test);
    else
        do
        local _fell_through = false
        local _result = {xpcall(function()
            test_environment:run(_.test);
            _fell_through = true
        end, enhance_error)}
        if _result[1] then
            -- Success:
    if not _fell_through then return table.unpack(_result, 2) end
        else
            -- Failure:
    
    msg = _result[2]
    src = Source:from_string(_.source);
    l1 = file:line_number_at(src.start);
    l2 = file:line_number_at(src.stop);
    failures:add(
        Text(yellow(Text(src.filename, ":", l1, "-", l2, ":")), "\n",
          bright(red(msg:indented()))));
        end
    end
    end
        end
    end -- for-loop;
    if ((#failures) == 0) then
        if args.v then
        say(green("PASS"));
    else
        say(Text("\013[", green("PASS")));
    end
    else
        if args.v then
        say(red(bright("FAIL")));
    else
        say(Text("\013[", red(bright("FAIL"))));
    end
    say(Text((failures:joined_with("\n")):indented()));
    end
    end
        end
    end -- for-loop;
end);
if WAS_RUN_DIRECTLY then
    run_with(COMMAND_LINE_ARGS);
end
